package net.wanji.common.action;

import net.wanji.common.annotation.ActionBean;
import net.wanji.common.annotation.ActionMethod;
import net.wanji.common.annotation.ActionMethodParameter;
import org.apache.commons.lang3.StringUtils;

import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.*;

/**
 * <>
 *
 * @author wanji
 * 2020/6/7 16:59
 */
@ActionBean(name = "日期")
public class DateAction {

    @ActionMethod(name = "当前日期字符串")
    @ActionMethodParameter(names = {})
    public static String getDateStr() {
        return format(new Date(), "yyyy-MM-dd");
    }

    @ActionMethod(name = "格式化当前日期字符串")
    @ActionMethodParameter(names = {"格式"})
    public static String getFormatDateStr(String format) {
        if (StringUtils.isBlank(format)) {
            format = "yyyy-MM-dd";
        }
        return format(new Date(), format);
    }

    @ActionMethod(name = "当前日期")
    @ActionMethodParameter(names = {})
    public static Date getDate() {
        return new Date();
    }

    @ActionMethod(name = "取指定月份天数")
    @ActionMethodParameter(names = {"开始日期", "结束日期", "月份"})
    public static int buildIncludeMonthDays(Object startDate, Object endDate, String months) {
        return DateAction.buildIncludeMonthDays(startDate, endDate, months, true);
    }

    @ActionMethod(name = "取非指定月份天数")
    @ActionMethodParameter(names = {"开始日期", "结束日期", "月份"})
    public int buildExcludeMonthDays(Object startDate, Object endDate, String months) {
        return DateAction.buildIncludeMonthDays(startDate, endDate, months, false);
    }

    @ActionMethod(name = "取指定月份天数")
    @ActionMethodParameter(names = {"开始日期", "结束日期", "月份"})
    private static int buildIncludeMonthDays(Object startDateObj, Object endDateObj, String month, boolean in) {
        if (Objects.isNull(startDateObj)) {
            throw new RuntimeException("开始日期不能为空！");
        }
        if (Objects.isNull(endDateObj)) {
            throw new RuntimeException("开始日期不能为空！");
        }
        Date startDate = DateAction.parse(startDateObj);
        Date endDate = DateAction.parse(endDateObj);
        if (startDate.compareTo(endDate) > 0) {
            throw new RuntimeException("开始日期必须要小于结束日期！");
        }
        int inDay = 0;
        int notInDay = 0;
        String[] months = month.split(",");
        List<String> monthList = Arrays.asList(months);
        Calendar calendar;
        for (; startDate.compareTo(endDate) <= 0; startDate = calendar.getTime()) {
            calendar = Calendar.getInstance();
            calendar.setTime(startDate);
            int startMonth = calendar.get(Calendar.MONTH) + 1;
            if (monthList.contains(String.valueOf(startMonth))) {
                if (in) {
                    ++inDay;
                }
            } else if (!in) {
                ++notInDay;
            }
            calendar.add(Calendar.DAY_OF_MONTH, 1);
        }

        if (in) {
            return inDay;
        } else {
            return notInDay;
        }
    }

    private static Date parse(Object obj) {
        SimpleDateFormat simpleDateFormat = new SimpleDateFormat("yyyy-MM-dd");
        if (obj instanceof Date) {
            Date date = (Date) obj;
            String format = simpleDateFormat.format(date);

            try {
                return simpleDateFormat.parse(format);
            } catch (ParseException e) {
                throw new RuntimeException(e);
            }
        } else {
            String str = obj.toString();

            try {
                return simpleDateFormat.parse(str);
            } catch (ParseException var8) {
                simpleDateFormat = new SimpleDateFormat("yyyy/MM/dd");

                try {
                    return simpleDateFormat.parse(str);
                } catch (ParseException e) {
                    throw new RuntimeException("不能将[" + e + "]解析成日期");
                }
            }
        }
    }

    @ActionMethod(name = "解析字符串为日期")
    @ActionMethodParameter(names = {"日期字符串", "格式"})
    public static Date formatString(String dateStr, String format) {
        if (StringUtils.isBlank(dateStr)) {
            return null;
        } else {
            SimpleDateFormat sdf = new SimpleDateFormat(format);

            try {
                return sdf.parse(dateStr);
            } catch (ParseException var5) {
                throw new RuntimeException(var5);
            }
        }
    }

    @ActionMethod(name = "格式化日期")
    @ActionMethodParameter(names = {"目标日期", "格式"})
    public static String format(Date date, String format) {
        if (date == null) {
            return null;
        } else {
            SimpleDateFormat var3 = new SimpleDateFormat(format);
            return var3.format(date);
        }
    }

    @ActionMethod(name = "加日期")
    @ActionMethodParameter(names = {"目标日期", "年数", "月数", "天数", "小时", "分钟", "秒数"})
    public static Date addDate(Date date, int year, int month, int day, int hour, int minute, int second) {
        if (date == null) {
            return null;
        } else {
            Calendar calendar = Calendar.getInstance();
            calendar.setTime(date);
            calendar.add(Calendar.YEAR, year);
            calendar.add(Calendar.MONTH, month);
            calendar.add(Calendar.DAY_OF_MONTH, day);
            calendar.add(Calendar.HOUR_OF_DAY, hour);
            calendar.add(Calendar.MINUTE, minute);
            calendar.add(Calendar.SECOND, second);
            return calendar.getTime();
        }
    }

    @ActionMethod(name = "日期加年")
    @ActionMethodParameter(names = {"目标日期", "年数"})
    public static Date addDateForYear(Date date, int year) {
        if (date == null) {
            return null;
        } else {
            Calendar calendar = Calendar.getInstance();
            calendar.setTime(date);
            calendar.add(Calendar.YEAR, year);
            return calendar.getTime();
        }
    }

    @ActionMethod(name = "日期加月")
    @ActionMethodParameter(names = {"目标日期", "月数"})
    public static Date addDateForMonth(Date date, int month) {
        if (date == null) {
            return null;
        } else {
            Calendar calendar = Calendar.getInstance();
            calendar.setTime(date);
            calendar.add(Calendar.MONTH, month);
            return calendar.getTime();
        }
    }

    @ActionMethod(name = "日期加天")
    @ActionMethodParameter(names = {"目标日期", "天数"})
    public static Date addDateForDay(Date date, int day) {
        if (date == null) {
            return null;
        } else {
            Calendar calendar = Calendar.getInstance();
            calendar.setTime(date);
            calendar.add(Calendar.DAY_OF_MONTH, day);
            return calendar.getTime();
        }
    }

    @ActionMethod(name = "日期加小时")
    @ActionMethodParameter(names = {"目标日期", "小时数"})
    public static Date addDateForHour(Date date, int hour) {
        if (date == null) {
            return null;
        } else {
            Calendar calendar = Calendar.getInstance();
            calendar.setTime(date);
            calendar.add(Calendar.HOUR_OF_DAY, hour);
            return calendar.getTime();
        }
    }

    @ActionMethod(name = "日期加分钟")
    @ActionMethodParameter(names = {"目标日期", "分钟数"})
    public static Date addDateForMinute(Date date, int minute) {
        if (date == null) {
            return null;
        } else {
            Calendar calendar = Calendar.getInstance();
            calendar.setTime(date);
            calendar.add(Calendar.MINUTE, minute);
            return calendar.getTime();
        }
    }

    @ActionMethod(name = "日期加秒")
    @ActionMethodParameter(names = {"目标日期", "秒数"})
    public static Date addDateForSecond(Date date, int second) {
        if (date == null) {
            return null;
        } else {
            Calendar calendar = Calendar.getInstance();
            calendar.setTime(date);
            calendar.add(Calendar.SECOND, second);
            return calendar.getTime();
        }
    }

    @ActionMethod(name = "减日期")
    @ActionMethodParameter(names = {"目标日期", "年数", "月数", "天数", "小时", "分钟", "秒数"})
    public static Date subDate(Date date, int year, int month, int day, int hour, int minute, int second) {
        if (date == null) {
            return null;
        } else {
            Calendar calendar = Calendar.getInstance();
            calendar.setTime(date);
            calendar.add(Calendar.YEAR, -year);
            calendar.add(Calendar.MONTH, -month);
            calendar.add(Calendar.DAY_OF_MONTH, -day);
            calendar.add(Calendar.HOUR_OF_DAY, -hour);
            calendar.add(Calendar.MINUTE, -minute);
            calendar.add(Calendar.SECOND, -second);
            return calendar.getTime();
        }
    }

    @ActionMethod(name = "减日期减年")
    @ActionMethodParameter(names = {"目标日期", "年数"})
    public static Date subDateForYear(Date date, int year) {
        if (date == null) {
            return null;
        } else {
            Calendar calendar = Calendar.getInstance();
            calendar.setTime(date);
            calendar.add(Calendar.YEAR, -year);
            return calendar.getTime();
        }
    }

    @ActionMethod(name = "减日期减月")
    @ActionMethodParameter(names = {"目标日期", "月数"})
    public static Date subDateForMonth(Date date, int month) {
        if (date == null) {
            return null;
        } else {
            Calendar calendar = Calendar.getInstance();
            calendar.setTime(date);
            calendar.add(Calendar.MONTH, -month);
            return calendar.getTime();
        }
    }

    @ActionMethod(name = "减日期减天")
    @ActionMethodParameter(names = {"目标日期", "天数"})
    public static Date subDateForDay(Date date, int day) {
        if (date == null) {
            return null;
        } else {
            Calendar calendar = Calendar.getInstance();
            calendar.setTime(date);
            calendar.add(Calendar.DAY_OF_MONTH, -day);
            return calendar.getTime();
        }
    }

    @ActionMethod(name = "减日期减小时")
    @ActionMethodParameter(names = {"目标日期", "小时"})
    public static Date subDateForHour(Date date, int hour) {
        if (date == null) {
            return null;
        } else {
            Calendar calendar = Calendar.getInstance();
            calendar.setTime(date);
            calendar.add(Calendar.HOUR_OF_DAY, -hour);
            return calendar.getTime();
        }
    }

    @ActionMethod(name = "减日期减分钟")
    @ActionMethodParameter(names = {"目标日期", "分钟"})
    public static Date subDateForMinute(Date date, int minute) {
        if (date == null) {
            return null;
        } else {
            Calendar calendar = Calendar.getInstance();
            calendar.setTime(date);
            calendar.add(Calendar.MINUTE, -minute);
            return calendar.getTime();
        }
    }

    @ActionMethod(name = "减日期减秒")
    @ActionMethodParameter(names = {"目标日期", "秒数"})
    public static Date subDateForSecond(Date date, int second) {
        if (date == null) {
            return null;
        } else {
            Calendar calendar = Calendar.getInstance();
            calendar.setTime(date);
            calendar.add(Calendar.SECOND, -second);
            return calendar.getTime();
        }
    }

    @ActionMethod(name = "取年份")
    @ActionMethodParameter(names = {"目标日期"})
    public static Integer getYear(Date date) {
        if (date == null) {
            return null;
        } else {
            Calendar calendar = Calendar.getInstance();
            calendar.setTime(date);
            return calendar.get(Calendar.YEAR);
        }
    }

    @ActionMethod(name = "取月份")
    @ActionMethodParameter(names = {"目标日期"})
    public Integer getMonth(Date date) {
        if (date == null) {
            return null;
        } else {
            Calendar calendar = Calendar.getInstance();
            calendar.setTime(date);
            return calendar.get(Calendar.MONTH);
        }
    }

    @ActionMethod(
            name = "取星期"
    )
    @ActionMethodParameter(
            names = {"目标日期"}
    )
    public static Integer getWeek(Date date) {
        if (date == null) {
            return null;
        } else {
            Calendar calendar = Calendar.getInstance();
            calendar.setTime(date);
            return calendar.get(Calendar.DAY_OF_WEEK);
        }
    }

    @ActionMethod(name = "取天")
    @ActionMethodParameter(names = {"目标日期"})
    public static Integer getDay(Date date) {
        if (date == null) {
            return null;
        } else {
            Calendar calendar = Calendar.getInstance();
            calendar.setTime(date);
            return calendar.get(Calendar.DAY_OF_MONTH);
        }
    }

    @ActionMethod(name = "取小时")
    @ActionMethodParameter(names = {"目标日期"})
    public static Integer getHour(Date date) {
        if (date == null) {
            return null;
        } else {
            Calendar calendar = Calendar.getInstance();
            calendar.setTime(date);
            return calendar.get(Calendar.HOUR_OF_DAY);
        }
    }

    @ActionMethod(name = "取分钟")
    @ActionMethodParameter(names = {"目标日期"})
    public static Integer getMinute(Date date) {
        if (date == null) {
            return null;
        } else {
            Calendar calendar = Calendar.getInstance();
            calendar.setTime(date);
            return calendar.get(Calendar.MINUTE);
        }
    }

    @ActionMethod(name = "取秒")
    @ActionMethodParameter(names = {"目标日期"})
    public static Integer getSecond(Date date) {
        if (date == null) {
            return null;
        } else {
            Calendar calendar = Calendar.getInstance();
            calendar.setTime(date);
            return calendar.get(Calendar.SECOND);
        }
    }

    @ActionMethod(name = "日期相减返回秒")
    @ActionMethodParameter(names = {"日期", "减去的日期"})
    public static Long dateDifSecond(Date startDate, Date endDate) {
        if (startDate != null && endDate != null) {
            Calendar startCalendar = Calendar.getInstance();
            startCalendar.setTime(startDate);
            Calendar endCalendar = Calendar.getInstance();
            endCalendar.setTime(endDate);
            long startMillis = startCalendar.getTimeInMillis();
            long endMillis = endCalendar.getTimeInMillis();
            return (startMillis - endMillis) / 1000L;
        } else {
            return null;
        }
    }

    @ActionMethod(
            name = "日期相减返回分钟"
    )
    @ActionMethodParameter(
            names = {"日期", "减去的日期"}
    )
    public static Long dateDifMinute(Date startDate, Date endDate) {
        if (startDate != null && endDate != null) {
            Calendar startCalendar = Calendar.getInstance();
            startCalendar.setTime(startDate);
            Calendar endCalendar = Calendar.getInstance();
            endCalendar.setTime(endDate);
            long startMillis = startCalendar.getTimeInMillis();
            long endMillis = endCalendar.getTimeInMillis();
            return (startMillis - endMillis) / 60000L;
        } else {
            return null;
        }
    }

    @ActionMethod(name = "日期相减返回小时")
    @ActionMethodParameter(names = {"日期", "减去的日期"})
    public static Long dateDifHour(Date startDate, Date endDate) {
        if (startDate != null && endDate != null) {
            Calendar startCalendar = Calendar.getInstance();
            startCalendar.setTime(startDate);
            Calendar endCalendar = Calendar.getInstance();
            endCalendar.setTime(endDate);
            long startMillis = startCalendar.getTimeInMillis();
            long endMillis = endCalendar.getTimeInMillis();
            return (startMillis - endMillis) / 3600000L;
        } else {
            return null;
        }
    }

    @ActionMethod(name = "日期相减返回天")
    @ActionMethodParameter(names = {"日期", "减去的日期"})
    public static Long dateDifDay(Date startDate, Date endDate) {
        if (startDate != null && endDate != null) {
            Calendar startCalendar = Calendar.getInstance();
            startCalendar.setTime(startDate);
            Calendar endCalendar = Calendar.getInstance();
            endCalendar.setTime(endDate);
            long startMillis = startCalendar.getTimeInMillis();
            long endMillis = endCalendar.getTimeInMillis();
            return (startMillis - endMillis) / 86400000L;
        } else {
            return null;
        }
    }

    @ActionMethod(
            name = "日期相减返回星期"
    )
    @ActionMethodParameter(
            names = {"日期", "减去的日期"}
    )
    public static Long dateDifWeek(Date startDate, Date endDate) {
        if (startDate != null && endDate != null) {
            Calendar startCalendar = Calendar.getInstance();
            startCalendar.setTime(startDate);
            Calendar endCalendar = Calendar.getInstance();
            endCalendar.setTime(endDate);
            long startMillis = startCalendar.getTimeInMillis();
            long endMillis = endCalendar.getTimeInMillis();
            return (startMillis - endMillis) / 604800000L;
        } else {
            return null;
        }
    }

    @ActionMethod(name = "日期相减返回月")
    @ActionMethodParameter(names = {"日期", "减去的日期"})
    public Integer dateDifMonth(Date startDate, Date endDate) {
        if (startDate != null && endDate != null) {
            Calendar startCalendar = Calendar.getInstance();
            startCalendar.setTime(startDate);
            Calendar endCalendar = Calendar.getInstance();
            endCalendar.setTime(endDate);
            int startYear = startCalendar.get(Calendar.YEAR);
            int endYear = endCalendar.get(Calendar.YEAR);
            int startMonth = startCalendar.get(Calendar.MONTH);
            int endMonth = endCalendar.get(Calendar.MONTH);
            return 12 * (startYear - endYear) + (startMonth - endMonth);
        } else {
            return null;
        }
    }

}
