package net.wanji.common.framework.config;

import net.wanji.common.framework.Constants;
import net.wanji.common.utils.tool.DateUtil;
import net.wanji.common.utils.file.FileUtil;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.InputStream;
import java.util.Properties;

/**
 * 资源文件帮助类，加载配置信息
 *
 * @author wanji
 * @date 2016/10/27
 */
@Slf4j
public class ConfigHelper {

    private static Properties properties;
    private static String filePath;

    private static final String PROPERTIES_FILE_NAME = "application.yml";

    static {
        System.out.println("[" + DateUtil.getDateTimeInSecond() + "] Loading application.yml");
        try {
            filePath = FileUtil.getFilePath(PROPERTIES_FILE_NAME, Constants.Env.BASE_HOME);
            if (filePath != null) {
                //系统参数配置
                properties = loadProperties();
            }
        } catch (Exception e) {
            log.error("配置文件读取失败！", e);
        }
    }

    private static Properties loadProperties() {
        Properties prop = new Properties();
        try {
            File file = FileUtil.getFile(ConfigHelper.PROPERTIES_FILE_NAME, Constants.Env.BASE_HOME);
            if (file != null) {
                InputStream input = new FileInputStream(file);
                prop.load(input);
            }
        } catch (Exception e) {
            log.error("Loading application.properties fails", e);
        }
        return prop;
    }

    /**
     * 根据Key  得到config文件中key对应的数据
     *
     * @param key
     * @return
     */
    public static String getValue(String key) {
        String value = null;
        try {
            value = properties.getProperty(key);
        } catch (Exception e) {
            log.error("key:" + key + " 资源参数加载失败！", e);
        }
        return value;
    }

    /**
     * @param key
     * @param value
     */
    public static void setProperties(String key, String value) {
        try {
            FileInputStream input = new FileInputStream(filePath);
            SafeProperties safeProp = new SafeProperties();
            safeProp.load(input);
            input.close();
            if (StringUtils.isNotEmpty(value)) {
                safeProp.put(key, value);
            }
            if (key != null) {
                if (StringUtils.isEmpty(value)) {
                    safeProp.remove(key);
                }
            }
            FileOutputStream output = new FileOutputStream(filePath);
            safeProp.store(output, null);
            output.close();
        } catch (Exception e) {
            log.error("Visit " + filePath + " for updating " + key + " value error", e);
        }
    }

    /**
     * 删除
     *
     * @param key
     */
    public static void removeProperties(String key) {
        try {
            FileInputStream input = new FileInputStream(filePath);
            SafeProperties safeProp = new SafeProperties();
            safeProp.load(input);
            input.close();
            if (key != null) {
                safeProp.remove(key);
            }
            FileOutputStream output = new FileOutputStream(filePath);
            safeProp.store(output, null);
            output.close();
        } catch (Exception e) {
            log.error("Visit " + filePath + " for updating " + key + " value error", e);
        }
    }
}
