package net.wanji.common.framework.db;

import net.wanji.common.framework.db.impl.MysqlDBImpl;
import net.wanji.common.framework.db.impl.OracleDBImpl;
import net.wanji.common.framework.db.impl.PostgresqlDBImpl;
import net.wanji.common.framework.db.impl.SQLServerDBImpl;

import static net.wanji.common.framework.Constants.DataBase.*;
import static net.wanji.common.framework.Constants.ServicePublishConstants.DBType.SQLSERVER;

/**
 * 数据库基础接口简单工厂类
 */
public class DbFactory {

    private DbFactory() {
    }

    private static class InnerClass {
        private static DbFactory dbBaseFactory = new DbFactory();
    }

    public static DbFactory getInstance() {
        return InnerClass.dbBaseFactory;
    }

    /**
     * 根据数据库类型 得到DBBaseInterface 的实现
     *
     * @param dbType 数据库连接信息。
     * @return DBBaseInterface 的实现。
     */
    public DbInterface getDbInterface(String dbType) {
        DbInterface bdBaseInterface;
        switch (dbType) {
            case MYSQL:
                bdBaseInterface = new MysqlDBImpl();
                break;
            case POSTGRESQL:
                bdBaseInterface = new PostgresqlDBImpl();
                break;
            case ORACLE:
                bdBaseInterface = new OracleDBImpl();
                break;
            case SQLSERVER:
                bdBaseInterface = new SQLServerDBImpl();
                break;
            default:
                bdBaseInterface = new MysqlDBImpl();
                break;
        }
        return bdBaseInterface;
    }
}
