package net.wanji.common.framework.domain;

import com.alibaba.fastjson.annotation.JSONField;
import io.swagger.annotations.ApiModelProperty;
import lombok.Getter;
import lombok.Setter;
import net.wanji.common.utils.tool.IDGenerator;
import org.apache.ignite.cache.query.annotations.QuerySqlField;

import java.io.Serializable;
import java.lang.reflect.Field;
import java.util.Date;
import java.util.HashMap;
import java.util.Map;

/**
 * 继承自该类的entity表示是可被跟踪的，可以查询对应实体
 *
 * @author wanji
 */
@Getter
@Setter
public class TrackableEntity implements Serializable {

    private static final long serialVersionUID = -4052705808523280313L;
    private static String ID = "id";
    private static String CREATE_DATE = "createDate";

    @QuerySqlField(index = true)
    //update时需校验非空
//    @NotBlank(groups = ValidationGroups.Update.class)
    public String id;

    @ApiModelProperty(value = "记录创建时间", example = "2016-08-01")
    @JSONField(format = "yyyy-MM-dd")
    @QuerySqlField(index = true)
    private Date createDate;
    @ApiModelProperty(value = "记录更新时间", example = "2016-08-01")
    @JSONField(format = "yyyy-MM-dd")
    private Date updateDate;

    @ApiModelProperty(value = "扩展字段，例如：{\"key1\":\"value1\",\"key2\":\"value2\"}")
    private Map<String, Object> expandMap;

    @ApiModelProperty(value = "扩充字段")
    private String expand;

    protected TrackableEntity() {
    }

    /**
     * 转换bean为map
     *
     * @param source 要转换的bean
     * @param <T>    bean类型
     * @return 转换结果
     */
    public static <T> Map<String, Object> bean2Map(T source) throws IllegalAccessException {
        Map<String, Object> result = new HashMap<>();

        Class<?> sourceClass = source.getClass();
        //拿到所有的字段,不包括继承的字段
        Field[] sourceFiled = sourceClass.getDeclaredFields();
        for (Field field : sourceFiled) {
            //设置可访问,不然拿不到private
            field.setAccessible(true);
            //配置了注解的话则使用注解名称,作为header字段
            FieldName fieldName = field.getAnnotation(FieldName.class);
            if (fieldName == null) {
                result.put(field.getName(), getSource(field, source));
            } else {
                if (fieldName.Ignore()) {
                    continue;
                }
                result.put(fieldName.value(), field.get(source));
            }
        }
        return result;
    }

    private static <T> Object getSource(Field field, T source) throws IllegalAccessException {
        if (field.get(source) != null) {
            return field.get(source);
        }
        if (ID.equals(field.getName())) {
            return IDGenerator.getID();
        }
        if (CREATE_DATE.equals(field.getName())) {
            return new Date();
        }
        return null;
    }
}
