package net.wanji.common.framework.dubbointerface;

import net.wanji.common.framework.domain.TrackableEntity;
import net.wanji.common.framework.exception.DubboProviderException;
import net.wanji.common.framework.rest.JsonViewObject;
import net.wanji.common.framework.rest.Page;

import java.util.List;
import java.util.Map;

/**
 * dubbo抽象接口，  所有dubbo的接口都需要继承
 *
 * @param <Entity>
 * @author wanji
 */
public interface BaseDubboInterface<Entity extends TrackableEntity> {

    /**
     * 分页查询
     *
     * @param page
     * @param map
     * @return
     * @throws DubboProviderException
     */
    Page<Entity> findByPage(Page<Entity> page, Map<String, Object> map) throws DubboProviderException;

    /**
     * 持久化对象的信息
     *
     * @param entity
     * @return
     * @throws DubboProviderException
     */
    JsonViewObject save(Entity entity) throws DubboProviderException;

    /**
     * 修改对象的信息
     *
     * @param entity
     * @return
     * @throws DubboProviderException
     */
    JsonViewObject update(Entity entity) throws DubboProviderException;

    /**
     * 根据ID 删除指定的对象
     *
     * @param id
     * @return
     * @throws DubboProviderException
     */
    JsonViewObject deleteById(String id) throws DubboProviderException;

    /**
     * 根据ID数组 删除指定的对象
     *
     * @param ids
     * @return
     * @throws DubboProviderException
     */
    JsonViewObject deleteByIds(String[] ids) throws DubboProviderException;

    /**
     * 根据ID 查找指定的对象
     *
     * @param id
     * @return
     * @throws DubboProviderException
     */
    Entity findById(String id) throws DubboProviderException;

    /**
     * 得到所有的对象
     *
     * @return
     * @throws DubboProviderException
     */
    List<Entity> findAll() throws DubboProviderException;

    /**
     * 用于多条件查询
     *
     * @param map
     * @return
     * @throws DubboProviderException
     */
    List<Entity> findByMap(Map<String, Object> map) throws DubboProviderException;

    /**
     * 操作日志记录
     * @param classNameCn
     * @param className
     * @param method
     * @param username
     * @param dbId
     * @param lastTime
     * @param jsonViewObject
     */
    void sendOperationLog(String classNameCn, String className, String method, String username, String dbId, long lastTime, JsonViewObject jsonViewObject);

}
