package net.wanji.common.framework.exception;

import net.wanji.common.framework.rest.JsonViewObject;
import org.springframework.validation.FieldError;
import org.springframework.web.bind.MethodArgumentNotValidException;
import org.springframework.web.bind.annotation.ExceptionHandler;
import org.springframework.web.bind.annotation.RestControllerAdvice;

import javax.validation.ConstraintViolation;
import javax.validation.ConstraintViolationException;
import java.util.List;
import java.util.stream.Collectors;

/**
 * @Description:
 * @Author Jack
 * @Date 2020/8/25 14:33
 */
@RestControllerAdvice
public class RestArgumentsExceptionHandler {
    //处理针对DTO的参数格式错误
    @ExceptionHandler(MethodArgumentNotValidException.class)
    public JsonViewObject handleMethodArgumentNotValidException(MethodArgumentNotValidException e) {
        List<FieldError> fieldErrors = e.getBindingResult().getFieldErrors();
        //一次性返回所有非法参数提示
        List<String> errors = fieldErrors.stream()
                .map(filedError -> String.format("参数【%s】不合法：%s",
                        filedError.getField(),
                        filedError.getDefaultMessage()))
                .collect(Collectors.toList());
        return JsonViewObject.newInstance().fail(errors, "请求参数格式错误！");
    }

    //处理RequestParam形式的参数校验错误
    @ExceptionHandler(ConstraintViolationException.class)
    public JsonViewObject handleMethodArgumentNotValidException(ConstraintViolationException e) {
        List<String> errors = e.getConstraintViolations().stream().map(ConstraintViolation::getMessage).collect(Collectors.toList());
        return JsonViewObject.newInstance().fail(String.join(",", errors), "请求参数格式错误！");
    }
}
