package net.wanji.common.framework.log;

import com.alibaba.fastjson.JSONObject;
import net.wanji.common.framework.Constants;
import net.wanji.common.framework.domain.OperationLog;
import net.wanji.common.framework.domain.TrackableEntity;
import net.wanji.common.framework.rest.JsonViewObject;
import net.wanji.common.tool.mqtool.MqUtil;
import eu.bitwalker.useragentutils.OperatingSystem;
import eu.bitwalker.useragentutils.UserAgent;
import lombok.extern.slf4j.Slf4j;
import org.springframework.web.context.request.RequestContextHolder;
import org.springframework.web.context.request.ServletRequestAttributes;

import javax.servlet.http.HttpServletRequest;
import javax.ws.rs.core.Context;
import java.util.Date;
import java.util.Map;
import java.util.Objects;

import static net.wanji.common.framework.Constants.LogType.OPERATION_LOG;

@Slf4j
public abstract class AbstractLog {

    @Context
    protected HttpServletRequest request;

    public void sendOperationLog(String classNameCn, String className, String method, String username, String dbId, long lastTime, JsonViewObject jsonViewObject) {
        OperationLog opLog = new OperationLog();
        try {
            opLog.setResName(classNameCn);
            opLog.setResType(className);
            opLog.setOperationType(method);
            opLog.setOperationUser(username);
            opLog.setDbId(dbId);
            opLog.setDes(Objects.isNull(jsonViewObject)?"": JSONObject.toJSONString(jsonViewObject));
            opLog.setOperationTime(lastTime);
            opLog.setStartTime(new Date());
            opLog.setEndTime(new Date());
            opLog.setStatus(Objects.isNull(jsonViewObject)? Constants.JsonView.STATUS_FAIL:jsonViewObject.getStatus());
            try {
                //用户客户端操作系统
                if (request == null) {
                    request = ((ServletRequestAttributes) RequestContextHolder.getRequestAttributes()).getRequest();
                }
                UserAgent userAgent = UserAgent.parseUserAgentString(request.getHeader("User-Agent"));
                //获取客户端操作系统
                OperatingSystem agentOs = userAgent.getOperatingSystem();
                //获取客户端浏览器
                String agentBrowser = "";
                if (null != userAgent.getBrowser() && null != userAgent.getBrowserVersion()) {
                    agentBrowser = userAgent.getBrowser().getName() + " " + userAgent.getBrowserVersion().getVersion();
                }
                opLog.setOperationOS(agentOs.getName());
                opLog.setOperationBrowser(agentBrowser);
                opLog.setClientParam(request.getHeader("User-Agent"));

                String agentIp = request.getHeader("X-Real-IP");
                if (agentIp == null || agentIp.length() == 0 || "unknown".equalsIgnoreCase(agentIp)) {
                    agentIp = request.getHeader("X-Forwarded-For");
                }
                if (agentIp == null || agentIp.length() == 0 || "unknown".equalsIgnoreCase(agentIp)) {
                    agentIp = request.getHeader("Proxy-Client-IP");
                }
                if (agentIp == null || agentIp.length() == 0 || "unknown".equalsIgnoreCase(agentIp)) {
                    agentIp = request.getHeader("WL-Proxy-Client-IP");
                }
                if (agentIp == null || agentIp.length() == 0 || "unknown".equalsIgnoreCase(agentIp)) {
                    agentIp = request.getRemoteAddr();
                }
                opLog.setOperationIp("0:0:0:0:0:0:0:1".equals(agentIp) ? "127.0.0.1" : agentIp);
            }catch (Exception ex){
                ex.printStackTrace();
            }
            MqUtil.sendLogMessage(OPERATION_LOG.toString(), OPERATION_LOG.toString(), TrackableEntity.bean2Map(opLog));
        } catch (Exception e) {
            log.error("SendOperationLog error", e);
        }
    }

    public static void getApiRequestLog(long start, String adCode, String version, String name, Map<String, Object> logDetailMap, HttpServletRequest request, String username) {
        long end = System.currentTimeMillis();
        if (request == null) {
            request = ((ServletRequestAttributes) RequestContextHolder.getRequestAttributes()).getRequest();
        }
        logDetailMap.put("adCode", adCode);
        logDetailMap.put("version", version);
        logDetailMap.put("name", name);
        logDetailMap.put("startTime", start);
        logDetailMap.put("endTime", end);
        logDetailMap.put("responseTime", String.valueOf(end - start));
        logDetailMap.put("createTime", System.currentTimeMillis());
        logDetailMap.put("queryParam", JSONObject.toJSONString(request.getParameterMap()));
        //username
        logDetailMap.put("username", username);
        //ip
        String agentIp="127.0.0.1";
        try {
            //用户客户端操作系统
            if (request != null) {
                agentIp = request.getHeader("X-Real-IP");
                if (agentIp == null || agentIp.length() == 0 || "unknown".equalsIgnoreCase(agentIp)) {
                    agentIp = request.getHeader("X-Forwarded-For");
                }
                if (agentIp == null || agentIp.length() == 0 || "unknown".equalsIgnoreCase(agentIp)) {
                    agentIp = request.getHeader("Proxy-Client-IP");
                }
                if (agentIp == null || agentIp.length() == 0 || "unknown".equalsIgnoreCase(agentIp)) {
                    agentIp = request.getHeader("WL-Proxy-Client-IP");
                }
                if (agentIp == null || agentIp.length() == 0 || "unknown".equalsIgnoreCase(agentIp)) {
                    agentIp = request.getRemoteAddr();
                }
            }
        }catch (Exception ex){
            ex.printStackTrace();
        }
        logDetailMap.put("ip","0:0:0:0:0:0:0:1".equals(agentIp) ? "127.0.0.1" : agentIp);
    }

}
