package net.wanji.common.framework.rest;

import lombok.Data;
import net.wanji.common.framework.Constants;
import net.wanji.common.framework.HttpCode;

import java.io.Serializable;

/**
 * restful对外的JSON 对象封装
 */
@Data
public class JsonViewObject implements Serializable {

    private Object content;
    private String message;
    private String status;
    private Integer code;
    private Long timestamp;

    public Object getContent() {
        return content;
    }

    public String getMessage() {
        return message;
    }

    public String getStatus() {
        return status;
    }

    private JsonViewObject() {
    }

    public static JsonViewObject newInstance() {
        return new JsonViewObject();
    }

    public JsonViewObject success() {
        return success((Object) "");
    }

    public JsonViewObject success(Object content) {
        return success(content, "");
    }

    public JsonViewObject success(String message) {
        return success("", message);
    }

    public JsonViewObject success(Object content, String message) {
        return pack(content, message, Constants.JsonView.STATUS_SUCCESS);
    }

    public JsonViewObject fail() {
        return fail("", "");
    }

    public JsonViewObject fail(Exception e) {
        String message = e.getMessage();
        int index = message.indexOf(":");
        return fail(index == -1 ? message : message.substring(index + 1));
    }

    public JsonViewObject fail(String errMsg) {
        return fail("", errMsg);
    }

    public JsonViewObject fail(Object content, String message) {
        return pack(content, message, Constants.JsonView.STATUS_FAIL);
    }

    private JsonViewObject pack(Object content, String message, String status) {
        this.content = content;
        this.message = message;
        this.status = status;
        //增加服务返回状态码，后期待完善
        this.code = (status.equals(Constants.JsonView.STATUS_SUCCESS) ? HttpCode.SUCCESS.value() : HttpCode.INTERNAL_SERVER_ERROR.value());
        return this;
    }
}
