package net.wanji.common.framework.rest;

import com.google.common.collect.Lists;
import com.google.common.collect.Maps;
import io.swagger.annotations.ApiModelProperty;
import lombok.Getter;
import lombok.Setter;

import java.io.Serializable;
import java.util.List;
import java.util.Map;

/**
 * @author
 * @date 2018/1/18
 */
@Getter
@Setter
public class NodeDTO implements Serializable {
    private static final long serialVersionUID = -3116372888315114025L;

    @ApiModelProperty(value = "节点ID")
    private String id;

    @ApiModelProperty(value = "父节点ID")
    private String pId;

    @ApiModelProperty(value = "节点名称")
    private String name;

    @ApiModelProperty(value = "子节点")
    private List<NodeDTO> children;

    @ApiModelProperty(value = "节点扩展属性")
    private Object data;

    @ApiModelProperty(value = "是否默认选中")
    private boolean checked;

    @ApiModelProperty(value = "是否禁用节点选择框")
    private boolean chkDisabled;

    @ApiModelProperty(value = "扩展节点标记")
    private boolean flag;

    @ApiModelProperty(value = "节点是否可操作，根据页面扩展属性")
    private boolean disabled;

    @ApiModelProperty(value = "节点是否打开")
    private boolean open;

    /**
     * 判断是否是其他节点,如是其他的为1,是树本身为0
     * 目前用到这一判断的地方为主对象分类挂载主对象
     */
    @ApiModelProperty(value = "判断是否是其他节点")
    private Integer isOtherBean;

    @ApiModelProperty(value = "经度")
    private Float lon;

    @ApiModelProperty(value = "纬度")
    private Float lat;
    @ApiModelProperty(value = "范围坐标")
    private String adrange;

    public NodeDTO(String id, String name, Object data, String pId) {
        this.id = id;
        this.name = name;
        this.data = data;
        this.pId = pId;
    }


    public NodeDTO(String id, String name, Object data, String pId,Float lon,Float lat) {
        this.id = id;
        this.name = name;
        this.data = data;
        this.pId = pId;
        this.lon=lon;
        this.lat=lat;
    }



    public NodeDTO(boolean checked, String id, String name, Object data, String pId) {
        this.checked = checked;
        this.id = id;
        this.name = name;
        this.data = data;
        this.pId = pId;
    }

    public NodeDTO(boolean checked, String id, String name, Object data, String pId, List<NodeDTO> children) {
        this.checked = checked;
        this.id = id;
        this.name = name;
        this.data = data;
        this.pId = pId;
        this.children = children;
    }

    public NodeDTO(boolean checked, boolean chkDisabled, String id, String name, Object data, String pId) {
        this.checked = checked;
        this.chkDisabled = chkDisabled;
        this.id = id;
        this.name = name;
        this.data = data;
        this.pId = pId;
    }

    public NodeDTO(String id, String name, Object data, String pId, boolean disabled) {
        this.id = id;
        this.name = name;
        this.data = data;
        this.pId = pId;
        this.disabled = disabled;
    }

    /**
     * 将Node集合[{id:1,pid:0},{id:2,pid:0},{id:0,pid:-1}]
     * 转换为[{id:0,pid:-1,children:[{id:1,pid:0},{id:2,pid:0}]}]
     *
     * @param nodeList node 集合
     * @return
     */
    public static List<NodeDTO> transformNodes(List<NodeDTO> nodeList) {
        List<NodeDTO> result = Lists.newArrayList();
        if (nodeList != null && !nodeList.isEmpty()) {
            Map<String, NodeDTO> nodeMap = Maps.newHashMap();
            nodeList.forEach(node -> nodeMap.put(node.getId(), node));
            nodeList.forEach(node -> {
                if (nodeMap.get(node.getPId()) != null) {
                    if (nodeMap.get(node.getPId()).getChildren() == null) {
                        nodeMap.get(node.getPId()).setChildren(Lists.newArrayList());
                    }
                    nodeMap.get(node.getPId()).getChildren().add(node);
                } else {
                    result.add(node);
                }
            });
        }
        return result;
    }

    /**
     * 将Node节点删去一个id=removeNodeId的子节点
     *
     * @param node
     * @return
     */
    public static NodeDTO removeNode(NodeDTO node, String removeNodeId) {
        if (node == null || removeNodeId.equals(node.getId())) {
            return null;
        }
        if (node.getChildren() == null || node.getChildren().isEmpty()) {
            return node;
        }
        List<NodeDTO> children = node.getChildren();
        node.setChildren(removeNode(children, removeNodeId));
        return node;
    }

    /**
     * 将Node集合删去一个id=removeNodeId的子节点
     *
     * @param nodeList
     * @return
     */
    public static List<NodeDTO> removeNode(List<NodeDTO> nodeList, String removeNodeId) {
        if (nodeList == null || nodeList.isEmpty()) {
            return Lists.newArrayList();
        }
        List<NodeDTO> result = Lists.newArrayList();
        for (NodeDTO node : nodeList) {
            if ((node = removeNode(node, removeNodeId)) != null) {
                result.add(node);
            }
        }
        return result;
    }

    /**
     * 将已转化好的树形结构的节点集nodeList，展开为平铺结构
     *
     * @param nodeList
     * @return List<Object>
     */
    public static List flatTree(List<NodeDTO> nodeList) {
        if (nodeList == null || nodeList.isEmpty()) {
            return Lists.newArrayList();
        }
        List<Object> result = Lists.newArrayList();
        for (NodeDTO node : nodeList) {
            result.addAll(flatTreeNode(node));
        }
        return result;
    }

    /**
     * 将单个节点，展开为平铺结构
     *
     * @param node
     * @return
     */
    public static List flatTreeNode(NodeDTO node) {
        if (node == null) {
            return Lists.newArrayList();
        }
        List<Object> result = Lists.newArrayList();
        result.add(node.getData());
        result.addAll(flatTree(node.getChildren()));
        return result;
    }

}
