package net.wanji.common.framework.rest;

import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;
import lombok.Getter;
import lombok.Setter;

import javax.validation.constraints.NotNull;
import javax.validation.constraints.PositiveOrZero;
import java.io.Serializable;
import java.util.List;

/**
 * {'total':64,'pageNum':1,'maxSize':10,'pageSize':10,'condition':''};
 *
 * @author wanji
 */
@Getter
@Setter
@ApiModel
public class Page<T> implements Serializable {
    /**
     * 每页数据条数
     */
    @ApiModelProperty(value = "每页显示的总条数", example = "10", dataType = "int")
    @PositiveOrZero //校验必须为正数或0
    private int pageSize = 10;
    /**
     * 总的数据数
     */
    @ApiModelProperty(value = "总的数据数", example = "10", dataType = "int")
    private int total = 0;
    /**
     * 起始页码
     */
    @ApiModelProperty(value = "当前页", example = "1", dataType = "int")
    @PositiveOrZero //校验必须为正数或0
    private int pageNum = 1;
    /**
     * 起始行数
     */
    private int startRowNum = 0;

    @NotNull
    private Object condition;

    private List<T> rows;

    public Page() {
    }

    public Page(int pageSize, int total, int pageNum) {
        this.pageSize = pageSize;
        this.total = total;
        this.pageNum = pageNum;
        this.startRowNum = pageSize * (pageNum - 1);
    }

    public int getTotalPageNum() {
        return (total + pageSize - 1) / pageSize;
    }

    public static Page newInstance() { return new Page(); }

    public static <T> Page<T> newInstance(int pageSize, int total, int pageNum) {
        return new Page<>(pageSize, total, pageNum);
    }

    /**
     * 开始内存分页
     * @param list
     * @param pageNum 页码
     * @param pageSize 每页多少条数据
     * @return
     */
    public static List startPage(List list, Integer pageNum,
                                 Integer pageSize) {
        if (list == null) {
            return null;
        }
        if (list.size() == 0) {
            return null;
        }

        Integer count = list.size(); // 记录总数
        Integer pageCount = 0; // 页数
        if (count % pageSize == 0) {
            pageCount = count / pageSize;
        } else {
            pageCount = count / pageSize + 1;
        }

        int fromIndex = 0; // 开始索引
        int toIndex = 0; // 结束索引

        if (pageNum != pageCount) {
            fromIndex = (pageNum - 1) * pageSize;
            toIndex = fromIndex + pageSize;
        } else {
            fromIndex = (pageNum - 1) * pageSize;
            toIndex = count;
        }

        List pageList = list.subList(fromIndex, toIndex);

        return pageList;
    }
}
