package net.wanji.common.framework.thread;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.concurrent.ThreadFactory;
import java.util.concurrent.atomic.AtomicInteger;

/**
 * 使用线程池时，根据业务名称来给池中线程命名。
 *
 * @Author wanji
 * @Date 2019/12/26 15:23
 */
public class NamedThreadFactory implements ThreadFactory {
    private static final AtomicInteger poolNumber = new AtomicInteger(1);

    private final ThreadGroup group;

    private final AtomicInteger threadNumber = new AtomicInteger(1);

    private final String namePrefix;

    private final boolean mDaemon;

    public NamedThreadFactory() {
        this("pool-" + poolNumber.getAndIncrement(), false);
    }

    public NamedThreadFactory(String threadName) {
        this(threadName, false);
    }

    public NamedThreadFactory(String threadName, boolean daemon) {
        SecurityManager s = System.getSecurityManager();
        group = (s != null) ? s.getThreadGroup() : Thread.currentThread().getThreadGroup();
        mDaemon = daemon;
        namePrefix = threadName + "-thread-";
    }

    @Override
    public Thread newThread(Runnable r) {
        String name = namePrefix + threadNumber.getAndIncrement();
        Thread t = new Thread(group, r, name, 0);
        t.setDaemon(mDaemon);
        t.setUncaughtExceptionHandler(new MyUncatchExceptionHandler());
        if (t.getPriority() != Thread.NORM_PRIORITY) {
            t.setPriority(Thread.NORM_PRIORITY);
        }
        return t;
    }
}

class MyUncatchExceptionHandler implements Thread.UncaughtExceptionHandler {
    private static final Logger log = LoggerFactory.getLogger(MyUncatchExceptionHandler.class);

    @Override
    public void uncaughtException(Thread t, Throwable e) {
        log.error("捕获到了线程池中[ threadName:{}, threadId:{} ] 的异常", t.getName(), t.getId(), e);
    }
}
