package net.wanji.common.utils;

import org.locationtech.jts.geom.Geometry;
import org.geotools.geojson.geom.GeometryJSON;
import org.locationtech.jts.geom.*;
import org.locationtech.jts.io.ParseException;
import org.locationtech.jts.io.WKTReader;

import java.io.StringWriter;
import java.math.BigDecimal;
import java.util.List;

public class FittingFunctionUntils {

    private static final GeometryFactory geometryFactory = new GeometryFactory();


    public static LineString calculateLineString(String wkt, int precision)
    {

        WKTReader reader = new WKTReader(geometryFactory);
        LineString line = null;
        try {
            line = (LineString)reader.read(wkt);
        } catch (ParseException e) {
            e.printStackTrace();
        }
        //LineString lineByWKT = GeometryUtils.createLineByWKT(wkt);
        return calculateLineString(line,precision);
    }

    public static LineString calculateLineString(LineString lineString, int precision) {
        double[][] points = new double[lineString.getNumPoints()][2];
        Coordinate[] coordinates = lineString.getCoordinates();
        for (int i = 0; i < coordinates.length; i++) {
            Coordinate coordinate = coordinates[i];
            points[i][0] = coordinate.x;
            points[i][1] = coordinate.y;
        }
        double[][] calculate = calculate(points, precision);
        Coordinate[] coordinatesNew = new Coordinate[calculate.length + 1];
        for (int i = 0; i < calculate.length; i++) {
            coordinatesNew[i] = new Coordinate(calculate[i][0], calculate[i][1]);
        }
        coordinatesNew[calculate.length] = lineString.getEndPoint().getCoordinate();

        return  geometryFactory.createLineString(coordinatesNew);
    }

    public static double[][] calculate(List<String> list, int precision) {
        double[][] points = new double[list.size()][2];
        for (int i = 0; i < list.size(); i++) {
            String point = (String)list.get(i);
            String[] split = point.split(",");
            points[i][0] = Double.parseDouble(split[0]);
            points[i][1] = Double.parseDouble(split[1]);
        }
        return calculate(points, precision);
    }

    public static double[][] calculate(double[][] poss, int precision)
    {
        int dimersion = poss[0].length;

        int number = poss.length;

        if ((number < 2) || (dimersion < 2)) {
            return (double[][])null;
        }
        double[][] result = new double[precision][dimersion];

        int[] mi = new int[number];
        int tmp43_42 = 1; mi[1] = tmp43_42; mi[0] = tmp43_42;
        for (int i = 3; i <= number; i++)
        {
            int[] t = new int[i - 1];
            for (int j = 0; j < t.length; j++)
                t[j] = mi[j];
            int tmp100_99 = 1; mi[(i - 1)] = tmp100_99; mi[0] = tmp100_99;
            for (int j = 0; j < i - 2; j++) {
                mi[(j + 1)] = (t[j] + t[(j + 1)]);
            }

        }

        for (int i = 0; i < precision; i++) {
            double t = new BigDecimal(i + "").divide(new BigDecimal(precision + ""), 80, 0).doubleValue();
            for (int j = 0; j < dimersion; j++) {
                double temp = 0.0D;
                for (int k = 0; k < number; k++) {
                    temp += Math.pow(1.0D - t, number - k - 1) * poss[k][j] * Math.pow(t, k) * mi[k];
                }
                result[i][j] = temp;
            }
        }

        return result;
    }


    public static  void main(String []args){


        LineString lineString = FittingFunctionUntils.calculateLineString("LINESTRING(118.700421223022 37.4278045360367,118.700798456007  37.4278045360367,118.700798456007 37.4276183268811)", 10);

        String json = null;
        try {
            org.locationtech.jts.io.WKTReader reader = new  org.locationtech.jts.io.WKTReader();
            Geometry geometry = reader.read(lineString.toText());
            StringWriter writer = new StringWriter();
            GeometryJSON g = new GeometryJSON(20);
            g.write(geometry, writer);
            json = writer.toString();
        } catch (Exception e) {
            e.printStackTrace();
        }

        System.out.println( json);

    }
}
