package net.wanji.common.utils.licenseUtils;

import lombok.extern.slf4j.Slf4j;

import java.io.*;
import java.net.InetAddress;
import java.net.NetworkInterface;
import java.util.Arrays;
import java.util.List;
import java.util.Objects;

import org.apache.commons.lang3.StringUtils;

@Slf4j
public class HardWareUtils {

    //安全性考虑，构造方法私有化
    private HardWareUtils() {}

    /**
     * @Description: 获取当前操作系统名称
     *
     * @return java.lang.String
     * @Author wanji
     * @Date 2020/7/28 17:45
     **/
    public static String getOSName() {
        return System.getProperty("os.name").toLowerCase();
    }

    /**
     * @Description: 判断当前操作系统
     *
     * @return boolean，true-unix系统，false，其他，本工具类内其他会按照windows系统处理
     * @Author wanji
     * @Date 2020/7/28 17:47
     **/
    public static boolean isUnixOs() {
        System.out.println(getOSName());
        return getOSName().contains("linux")||getOSName().contains("mac")||getOSName().contains("unix");
    }

    /**
     * @Description: 获取主板序列号,windows实现
     *
     * @return java.lang.String
     * @Author wanji
     * @Date 2020/7/28 17:53
     **/
    public static String getDosMotherboardSN() throws IOException {
        StringBuffer result = new StringBuffer();
        String vbs =
                "Set objWMIService = GetObject(\"winmgmts:\\\\.\\root\\cimv2\")\n"
                        + "Set colItems = objWMIService.ExecQuery _ \n"
                        + "   (\"Select * from Win32_BaseBoard\") \n"
                        + "For Each objItem in colItems \n"
                        + "    Wscript.Echo objItem.SerialNumber \n"
                        + "    exit for  ' do the first cpu only! \n"
                        + "Next \n";
        exeDosCmd(vbs).forEach(item -> {
            if (Objects.nonNull(item) && StringUtils.isNotBlank(item.toString()))
                result.append(item.toString().trim());
        });
        return result.toString();
    }

    /**
     * @Description: 获取unix系统 主板序列号
     *
     * @return java.lang.String
     * @Author wanji
     * @Date 2020/7/28 20:26
     **/
    public static String getUnixMotherboardSN() throws IOException {
        StringBuffer stringBuffer = new StringBuffer();
        String maniBordCmd = "dmidecode | grep 'Serial Number' | awk '{print $3}' | tail -1";
        exeShellCmd(maniBordCmd).forEach(item -> {
            if (Objects.nonNull(item) && StringUtils.isNotBlank(item.toString())) {
                stringBuffer.append(item.toString());
            }
        });
        return stringBuffer.toString();
    }

    /**
     * @Description: 获取unix系统 cpu序列号
     *
     * @return java.lang.String
     * @Author wanji
     * @Date 2020/7/28 19:41
     **/
    public static String getUnixCPUSerial() throws IOException {
        StringBuffer result = new StringBuffer();
        exeShellCmd("dmidecode").forEach(item -> {
            if (Objects.nonNull(item) &&
                    StringUtils.isNotBlank(item.toString()) &&
                    StringUtils.containsIgnoreCase(item.toString(), "uuid")) {
                result.append(StringUtils.substringAfter(item.toString(), "uuid").trim());
            }
        });
        return result.toString();
    }

    /**
     * @Description: 执行unix shell脚本
     *
     * @param cmd 命令
     * @return java.util.List<java.lang.Object>
     * @Author wanji
     * @Date 2020/7/28 20:30
     **/
    private static List<Object> exeShellCmd(String cmd) throws IOException {
        List<Object> result;
        Process p;
        p = Runtime.getRuntime().exec(new String[] { "sh", "-c", cmd }); //管道
        BufferedReader bufferedReader = new BufferedReader(new InputStreamReader(p.getInputStream()));
        result = Arrays.asList(bufferedReader.lines().toArray());
        bufferedReader.close();
        return result;
    }

    /**
     * @Description: 执行dos脚本
     *
     * @param cmd 命令
     * @return java.util.List<java.lang.Object>
     * @Author wanji
     * @Date 2020/7/28 20:25
     **/
    private static List<Object> exeDosCmd(String cmd) throws IOException {
        List<Object> result;
        File file = File.createTempFile("tmp", ".vbs");
        file.deleteOnExit();
        FileWriter fw = new FileWriter(file);
        fw.write(cmd);
        fw.close();
        Process p = Runtime.getRuntime().exec("cscript //NoLogo " + file.getPath());
        BufferedReader input = new BufferedReader(new InputStreamReader(p.getInputStream()));
        result = Arrays.asList(input.lines().toArray());
        input.close();
        file.delete();
        return result;
    }

    /**
     * @Description: 获取CPU序列号
     *
     * @return java.lang.String
     * @Author wanji
     * @Date 2020/7/28 17:53
     **/
    public static String getDosCPUSerial() throws IOException {
        StringBuffer result = new StringBuffer();
        String vbs =
                "Set objWMIService = GetObject(\"winmgmts:\\\\.\\root\\cimv2\")\n"
                + "Set colItems = objWMIService.ExecQuery _ \n"
                + "   (\"Select * from Win32_Processor\") \n"
                + "For Each objItem in colItems \n"
                + "    Wscript.Echo objItem.ProcessorId \n"
                + "    exit for  ' do the first cpu only! \n"
                + "Next \n";
        exeDosCmd(vbs).forEach(item -> {
            if (Objects.nonNull(item) && StringUtils.isNotBlank(item.toString()))
            result.append(item.toString().trim());
        });
        return result.toString();
    }

    /**
     * @Description: 获取MAC地址，多网卡环境存在计算不准的问题
     *
     * @return java.lang.String
     * @Author wanji
     * @Date 2020/7/28 19:46
     **/
    public static String getMac() {
        try {
            byte[] mac =
                    NetworkInterface.getByInetAddress(InetAddress.getLocalHost())
                            .getHardwareAddress();
            StringBuffer sb = new StringBuffer();
            for (int i = 0; i < mac.length; i++) {
                if (i != 0) {
                    sb.append("-");
                }
                String s = Integer.toHexString(mac[i] & 0xFF);
                sb.append(s.length() == 1 ? 0 + s : s);
            }
            return sb.toString().toUpperCase();
        } catch (Exception e) {
            return "";
        }
    }

    /**
     * @Description: 获取主板序列号，支持unix和dos系统
     *
     * @return java.lang.String
     * @Author wanji
     * @Date 2020/7/28 20:25
     **/
    public static String getMotherboardSN() throws IOException {
        return isUnixOs() ? getUnixMotherboardSN() : getDosMotherboardSN();
    }

    /**
     * @Description: 获取cpu序列号，支持unix和dos系统
     *
     * @return java.lang.String
     * @Author wanji
     * @Date 2020/7/28 20:24
     **/
    public static String getCPUSerial() throws IOException {
        return isUnixOs() ? getUnixCPUSerial() : getDosCPUSerial();
    }

    public static void main(String[] args) throws IOException {
        System.out.println("CPU SN: " + HardWareUtils.getCPUSerial());
        System.out.println("主板 SN: " + HardWareUtils.getMotherboardSN());
        System.out.println("MAC SN: " + HardWareUtils.getMac());
        try {
            System.out.println(DESUtils.encrypt(HardWareUtils.getCPUSerial() + "," + HardWareUtils.getMotherboardSN() +","+ HardWareUtils.getMac() +","+ "2020-08-05 23:59:59"));
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    /**
     * @Author wanji
     * @Description 获取机器码
     * @Date 18:49 2020/8/3
     * @Param []
     * @return java.lang.String
     * @throws
     */
    public static String getMachineCode() throws IOException{
        return getCPUSerial() + "-" + getMotherboardSN() ;
    }
}
