package net.wanji.common.utils.tool;

import com.csvreader.CsvReader;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang.StringEscapeUtils;

import java.io.*;
import java.nio.charset.StandardCharsets;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * @ClassName CsvUtils
 * @Description
 * @Author wanji
 * @Date 2020/7/15 11:44
 * @Version 1.0
 */
@Slf4j
public class CsvUtils {

    private static String split = ",";


    /**
     * @return void
     * @throws
     * @Author wanji
     * @Description 写入标题
     * @Date 15:01 2020/7/15
     * @Param [header, file]
     */
    public static void writeTitle(List<String> header, File file) throws Exception {
        BufferedWriter bw = null;
        try {
            bw = new BufferedWriter(new OutputStreamWriter(new FileOutputStream(file), StandardCharsets.UTF_8));
            // 填充对象属性值
            if (!CollectionUtils.isEmpty(header)) {
                // 添加头部（首行）
                for (int i = 0; i < header.size(); i++) {
                    bw.append(header.get(i));
                    if (i != header.size() - 1) {
                        bw.append(split);
                    }
                }
                bw.newLine();
            }
        } catch (Exception e) {
            log.error("Write CSV exception------", e);
            throw e;
        } finally {
            if (bw != null) {
                try {
                    bw.close();
                } catch (IOException e) {
                    log.error("Close OutputStream exception------", e);
                }
            }
        }
    }

    /**
     * @return void
     * @throws
     * @Author wanji
     * @Description 填充数据
     * @Date 15:01 2020/7/15
     * @Param [data, header, file]
     */
    public static void writeCSV(List<Map<String, Object>> data, List<String> header, File file) throws Exception {
        BufferedWriter bw = null;
        try {
            bw = new BufferedWriter(new OutputStreamWriter(new FileOutputStream(file, true), StandardCharsets.UTF_8));
            // 填充数据
            if (!CollectionUtils.isEmpty(data)) {
                for (Map<String, Object> map : data) {
                    for (int j = 0; j < header.size(); j++) {
                        String str = StringEscapeUtils.escapeCsv(String.valueOf(map.get(header.get(j))));
                        bw.append(str == null ? "" : str);
                        if (j != header.size() - 1) {
                            bw.append(split);
                        }
                    }
                    bw.newLine();
                }
            }
        } catch (Exception e) {
            log.error("Write CSV exception------", e);
            throw e;
        } finally {
            if (bw != null) {
                try {
                    bw.close();
                } catch (IOException e) {
                    log.error("Close OutputStream exception------", e);
                }
            }
        }
    }

    public static List<Map<String, Object>> readCSV(File file) throws Exception {
        List<Map<String, Object>> list = new ArrayList<>();
        try {
            CsvReader reader = new CsvReader(file.getPath(), ',', StandardCharsets.UTF_8);
            //关闭安全性校验，否则大字段读取会失败,http://javacsv.sourceforge.net/com/csvreader/CsvReader.html
            reader.setSafetySwitch(false);
            // 跳过表头 如果需要表头的话，不要写这句
            reader.readHeaders();
            String[] title = reader.getHeaders();
            while (reader.readRecord()) {
                String[] values = reader.getValues();
                Map<String, Object> map = new HashMap<>();
                for (int i = 0; i < title.length; i++) {
                    String s = StringEscapeUtils.unescapeCsv(values[i]);
                    if (StringUtils.isBlank(s) || StringUtils.equalsIgnoreCase(s, "null")) {
                        s = null;
                    }
                    map.put(title[i], s);
                }
                list.add(map);
            }
            reader.close();
        } catch (IOException e) {
            e.printStackTrace();
            log.error("read csv file error. : {}", e.getLocalizedMessage());
        }
        return list;
    }
}
