package net.wanji.common.utils.webservice;

import net.wanji.common.framework.exception.DubboProviderException;
import lombok.extern.slf4j.Slf4j;
import org.apache.http.HttpEntity;
import org.apache.http.client.config.RequestConfig;
import org.apache.http.client.methods.CloseableHttpResponse;
import org.apache.http.client.methods.HttpPost;
import org.apache.http.entity.StringEntity;
import org.apache.http.impl.client.CloseableHttpClient;
import org.apache.http.impl.client.HttpClientBuilder;
import org.apache.http.util.EntityUtils;

import java.nio.charset.Charset;
import java.util.HashMap;
import java.util.Map;

/**
 * Created by wanji on 2019/5/22.
 */
@Slf4j
public class WebServiceUtils {
    // 请求超时时间
    static int socketTimeout = 30000;
    // 传输超时时间
    static int connectTimeout = 30000;

    /**
     * 使用SOAP1.1发送消息
     *
     * @param postUrl
     * @param soapXml
     * @return
     */
    public static String doPostSoap1_1(String postUrl, String soapXml, Map<String, Object> headerMap) throws DubboProviderException {
        String retStr = "";
        // 创建HttpClientBuilder
        HttpClientBuilder httpClientBuilder = HttpClientBuilder.create();
        // HttpClient
        CloseableHttpClient closeableHttpClient = httpClientBuilder.build();
        HttpPost httpPost = new HttpPost(postUrl);
        //  设置请求和传输超时时间
        RequestConfig requestConfig = RequestConfig.custom()
                .setSocketTimeout(socketTimeout)
                .setConnectTimeout(connectTimeout).build();
        httpPost.setConfig(requestConfig);
        try {
            // httpPost.setHeader("Content-Type", "text/xml;charset=UTF-8");
            httpPost.setHeader("Content-Type", String.valueOf(headerMap.get("Content-Type")));
            StringEntity data = new StringEntity(soapXml,
                    Charset.forName("UTF-8"));
            httpPost.setEntity(data);
            CloseableHttpResponse response = closeableHttpClient
                    .execute(httpPost);
            HttpEntity httpEntity = response.getEntity();
            if (httpEntity != null) {
                // 打印响应内容
                retStr = EntityUtils.toString(httpEntity, "UTF-8");
                log.info("response:" + retStr);
            }
            // 释放资源
            closeableHttpClient.close();
        } catch (Exception e) {
            log.error("exception in doPostSoap1_1", e);
            throw new DubboProviderException(e);
        }
        return retStr;
    }

    /**
     * 使用SOAP1.2发送消息
     *
     * @param postUrl
     * @param soapXml
     * @param soapAction
     * @return
     */
    public static String doPostSoap1_2(String postUrl, String soapXml,
                                       String soapAction, Map<String, Object> headerMap) {
        String retStr = "";
        // 创建HttpClientBuilder
        HttpClientBuilder httpClientBuilder = HttpClientBuilder.create();
        // HttpClient
        CloseableHttpClient closeableHttpClient = httpClientBuilder.build();
        HttpPost httpPost = new HttpPost(postUrl);
        // 设置请求和传输超时时间
        RequestConfig requestConfig = RequestConfig.custom()
                .setSocketTimeout(socketTimeout)
                .setConnectTimeout(connectTimeout).build();
        httpPost.setConfig(requestConfig);
        try {
//            httpPost.setHeader("Content-Type",
//                    "application/soap+xml;charset=UTF-8");
            httpPost.setHeader("Content-Type", String.valueOf(headerMap.get("Content-Type")));
            httpPost.setHeader("SOAPAction", soapAction);
            StringEntity data = new StringEntity(soapXml,
                    Charset.forName("UTF-8"));
            httpPost.setEntity(data);
            CloseableHttpResponse response = closeableHttpClient
                    .execute(httpPost);
            HttpEntity httpEntity = response.getEntity();
            if (httpEntity != null) {
                // 打印响应内容
                retStr = EntityUtils.toString(httpEntity, "UTF-8");
                log.info("response:" + retStr);
            }
            // 释放资源
            closeableHttpClient.close();
        } catch (Exception e) {
            log.error("exception in doPostSoap1_2", e);
        }
        return retStr;
    }

    public static void main(String[] args) {
        String url = "http://192.168.10.179:8080/helloService?wsdl";
        String targetNamespace = "http://webservice.restful.design.bp.com/";
        String funName = "sayHello";
        Map bodyMap = new HashMap() {{
            put("message", "你好");
        }};
        String xml = SoapUtils.buildSoapXml(targetNamespace, funName, bodyMap);
        System.out.println(xml);
        try {
            System.out.println(doPostSoap1_1(url, xml, new HashMap<String, Object>() {{
                put("Content-Type", "text/xml; charset=utf-8");
            }}));
        } catch (DubboProviderException e) {
            e.printStackTrace();
        }
    }
}
